//
// You are free to use this code in part or whole in any way you seem fit!
// If you find this code useful we would to know how,
// you can find us at www.spellofplay.com feel free to drop by!
//

texture g_texture0 : texture0;

float g_resolutionX : editable;
float g_resolutionY : editable;



sampler g_diffuseMap = sampler_state {
   Texture = (g_texture0);
   MAGFILTER = LINEAR;
   MINFILTER = LINEAR;
   AddressU = CLAMP;
   AddressV = CLAMP;
};

struct PERPIXEL_VS_OUTPUT { 
	float4 m_pos   : POSITION;
	float2 m_uv		: TEXCOORD0;
	float4 m_color	: COLOR0;
};

PERPIXEL_VS_OUTPUT PerPixel_VS_BG(float4 a_pos  : POSITION,  float2 a_trans : TEXCOORD0, float3 a_scaleRot : TEXCOORD1, float2 a_uv : TEXCOORD2, float4 a_color : COLOR0) {
    PERPIXEL_VS_OUTPUT Out = (PERPIXEL_VS_OUTPUT) 0; 
    // Output transformed position:
   
    a_scaleRot.xy *= 3.0f;
   
    float c = cos(a_scaleRot.z - 3.1415 * 0.5f);
    float s = sin(a_scaleRot.z - 3.1415 * 0.5f);
    
    Out.m_pos.y = c * a_pos.x - s * a_pos.y;
    Out.m_pos.x = s * a_pos.x + c * a_pos.y;
    
    Out.m_pos.xy *= a_scaleRot.xy;
    Out.m_pos.xy += a_trans;
   
    // transform to normalized space
    Out.m_pos.x = Out.m_pos.x / (g_resolutionX * 0.5f) - 1.0f;
    Out.m_pos.y = 1.0 - (Out.m_pos.y / (g_resolutionY * 0.5f));
    
    Out.m_pos.z = 0.0f;
    Out.m_pos.w = 1.0f;
    Out.m_uv = a_uv;
    Out.m_color.r = a_color.b;
    Out.m_color.g = a_color.g;
    Out.m_color.b = a_color.r;
    Out.m_color.a = a_color.a;
    
    Out.m_color.rgb *= 0.15;
    
    
    return Out;
}

PERPIXEL_VS_OUTPUT PerPixel_VS_Main(float4 a_pos  : POSITION,  float2 a_trans : TEXCOORD0, float3 a_scaleRot : TEXCOORD1, float2 a_uv : TEXCOORD2, float4 a_color : COLOR0) {
   PERPIXEL_VS_OUTPUT Out = (PERPIXEL_VS_OUTPUT) 0; 

   // Output transformed position:
   
    float c = cos(a_scaleRot.z - 3.1415 * 0.5f);
    float s = sin(a_scaleRot.z - 3.1415 * 0.5f);
    
    Out.m_pos.y = c * a_pos.x - s * a_pos.y;
    Out.m_pos.x = s * a_pos.x + c * a_pos.y;
    
    Out.m_pos.xy *= a_scaleRot.xy;
    Out.m_pos.xy += a_trans;
   
    // transform to normalized space
    Out.m_pos.x = Out.m_pos.x / (g_resolutionX * 0.5f) - 1.0f;
    Out.m_pos.y = 1.0 - (Out.m_pos.y / (g_resolutionY * 0.5f));
    
    Out.m_pos.z = 0.0f;
    Out.m_pos.w = 1.0f;
    Out.m_uv = a_uv;
    Out.m_color.r = a_color.b;
    Out.m_color.g = a_color.g;
    Out.m_color.b = a_color.r;
    Out.m_color.a = a_color.a;
    
    return Out;
}

float4 PerPixel_PS_Main(	float4 a_color : COLOR0,
									float2 a_uv : TEXCOORD0) : COLOR { 

   float4 finalColor = tex2D(g_diffuseMap, a_uv);
   finalColor.rgb *= a_color.rgb;
   finalColor.rgb += finalColor.a;
   finalColor.rgb *= a_color.a;
   finalColor.a = 1.0f;
   
   return finalColor;
}

float4 PerPixel_PS_BG(	float4 a_color : COLOR0,
									float2 a_uv : TEXCOORD0) : COLOR { 

   float4 finalColor = tex2D(g_diffuseMap, a_uv);
   finalColor.rgb *= a_color.rgb;
   finalColor.a = 1.0f;
   
   return finalColor;
}

technique PerPixel {

   /*pass Single_Pass {
   	ZFunc = ALWAYS;
   	ZEnable = FALSE;
   	ZWriteEnable = FALSE;
      SrcBlend = ZERO;
      DestBlend = INVSRCCOLOR;
      FogEnable = FALSE;
      Lighting = FALSE;
      AlphaBlendEnable = TRUE;
      CullMode = NONE;
      VertexShader = NULL;
      VertexShader = compile vs_1_1 PerPixel_VS_BG();
      PixelShader = compile ps_2_0 PerPixel_PS_BG();
   }*/

   pass Single_Pass {
   	ZFunc = ALWAYS;
   	ZEnable = FALSE;
   	ZWriteEnable = FALSE;
      SrcBlend = ONE;
      DestBlend = INVSRCCOLOR;
      FogEnable = FALSE;
      Lighting = FALSE;
      AlphaBlendEnable = TRUE;
      CullMode = NONE;
      VertexShader = NULL;
      VertexShader = compile vs_1_1 PerPixel_VS_BG();
      PixelShader = compile ps_2_0 PerPixel_PS_BG();
   }

   pass Single_Pass {
   	ZFunc = ALWAYS;
   	ZEnable = FALSE;
   	ZWriteEnable = FALSE;
      SrcBlend = ONE;
      DestBlend = ONE;
      FogEnable = FALSE;
      Lighting = FALSE;
      AlphaBlendEnable = TRUE;
      CullMode = NONE;
      VertexShader = NULL;
      VertexShader = compile vs_1_1 PerPixel_VS_Main();
      PixelShader = compile ps_2_0 PerPixel_PS_Main();
   }
}


